@@ -4,7 +4,7 @@
 """
 ===============================================================================
 
-    Copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
+    Original code copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
 
     This file is part of cardinal_pythonlib.
 
@@ -21,6 +21,9 @@
     limitations under the License.
 
 ===============================================================================
+
+**Functions to assist with SQLAlchemy logs.**
+
 """
 
 import logging
@@ -28,15 +31,22 @@ import logging
 
 def pre_disable_sqlalchemy_extra_echo_log() -> None:
     """
-    If you create an SQLAlchemy Engine with echo=True, it creates an additional
-    log to stdout, via
+    Adds a null handler to the log named ``sqlalchemy.engine.base.Engine``,
+    which prevents a duplicated log stream being created later.
+
+    Why is this necessary?
+
+    If you create an SQLAlchemy :class:`Engine` with ``echo=True``, it creates
+    an additional log to ``stdout``, via
+
+    .. code-block:: python
 
         sqlalchemy.engine.base.Engine.__init__()
         sqlalchemy.log.instance_logger()
         sqlalchemy.log.InstanceLogger.__init__()
-            ... which checks that the logger has no handlers and if not calls:
+            # ... which checks that the logger has no handlers and if not calls:
         sqlalchemy.log._add_default_handler()
-            ... which adds a handler to sys.stdout
+            # ... which adds a handler to sys.stdout
     """
     log = logging.getLogger("sqlalchemy.engine.base.Engine")
     log.addHandler(logging.NullHandler())
